@@ -17,6 +17,7 @@ import tempfile
 QUOTE_SEPARATION = re.compile(r"(.*?)('.*?')", re.DOTALL)
 EXTRACT_EPOCH_PATTERN = re.compile(r'EXTRACT\s*\(\s*EPOCH\s+FROM',
         re.I)
+CAST_VARCHAR_PATTERN = re.compile(r'CAST\s*\((.*)AS VARCHAR\)', re.I)
 DatabaseIntegrityError = None
 DatabaseOperationalError = None
 
@@ -146,7 +147,7 @@ class Database(DatabaseInterface):
         for db_name, in cursor.fetchall():
             try:
                 database = Database(db_name).connect()
-            except:
+            except Exception:
                 continue
             cursor2 = database.cursor()
             if cursor2.test():
@@ -237,6 +238,8 @@ class Cursor(CursorInterface):
             nquote = nquote.replace('ilike', 'like')
             nquote = re.sub(EXTRACT_EPOCH_PATTERN, r'UNIX_TIMESTAMP(',
                     nquote)
+            nquote = re.sub(CAST_VARCHAR_PATTERN, r'CAST(\1AS CHAR)',
+                    nquote)
             buf += nquote + quote
         sql = buf[:-2]
         try:
