@@ -10,6 +10,10 @@ from psycopg2.extensions import ISOLATION_LEVEL_SERIALIZABLE
 from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
 from psycopg2.extensions import register_type, register_adapter
 from psycopg2.extensions import UNICODE, AsIs
+try:
+    from psycopg2.extensions import PYDATE, PYDATETIME, PYTIME
+except ImportError:
+    PYDATE, PYDATETIME, PYTIME = None, None, None
 from psycopg2 import IntegrityError as DatabaseIntegrityError
 from psycopg2 import OperationalError as DatabaseOperationalError
 import time
@@ -19,6 +23,7 @@ import os
 if os.name == 'posix':
     import pwd
 from decimal import Decimal
+from trytond.protocols.datatype import Float
 
 RE_FROM = re.compile('.* from "?([a-zA-Z_0-9]+)"?.*$')
 RE_INTO = re.compile('.* into "?([a-zA-Z_0-9]+)"?.*$')
@@ -191,7 +196,7 @@ class Database(DatabaseInterface):
             db_name = db_name.encode('utf-8')
             try:
                 database = Database(db_name).connect()
-            except:
+            except Exception:
                 continue
             cursor2 = database.cursor()
             if cursor2.test():
@@ -290,7 +295,7 @@ class Cursor(CursorInterface):
                 res = self.cursor.execute(sql, params)
             else:
                 res = self.cursor.execute(sql)
-        except:
+        except Exception:
             logger = logging.getLogger('sql')
             logger.error('Wrong SQL: ' + (self.cursor.query or ''))
             raise
@@ -395,6 +400,13 @@ class Cursor(CursorInterface):
         return select
 
 register_type(UNICODE)
+if PYDATE:
+    register_type(PYDATE)
+if PYDATETIME:
+    register_type(PYDATETIME)
+if PYTIME:
+    register_type(PYTIME)
 register_adapter(Session, AsIs)
 register_adapter(float, lambda value: AsIs(repr(value)))
+register_adapter(Float, lambda value: AsIs(repr(value)))
 register_adapter(Decimal, lambda value: AsIs(str(value)))
