@@ -264,7 +264,7 @@ class TableHandler(TableHandlerInterface):
                 # Populate column with default values:
                 default = None
                 if default_fun is not None:
-                    default = default_fun(self.cursor, 0, {})
+                    default = default_fun()
                 self.cursor.execute('UPDATE "' + self.table_name + '" '\
                                     'SET "' + column_name + '" = %s',
                                     (column_format(default),))
@@ -380,7 +380,7 @@ class TableHandler(TableHandlerInterface):
             self.cursor.execute('ALTER TABLE "%s" ' \
                            'ADD CONSTRAINT "%s" %s' % \
                            (self.table_name, ident, constraint,))
-        except:
+        except Exception:
             if exception:
                 raise
             logging.getLogger('init').warning(
@@ -400,7 +400,7 @@ class TableHandler(TableHandlerInterface):
             self.cursor.execute('ALTER TABLE "%s" ' \
                     'DROP CONSTRAINT "%s"' % \
                     (self.table_name, ident))
-        except:
+        except Exception:
             if exception:
                 raise
             logging.getLogger('init').warning(
@@ -416,7 +416,7 @@ class TableHandler(TableHandlerInterface):
                 'ALTER TABLE "%s" DROP COLUMN "%s"' %
                 (self.table_name, column_name))
 
-        except:
+        except Exception:
             if exception:
                 raise
             logging.getLogger('init').warning(
