@@ -1,12 +1,14 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-"Action"
-from trytond.model import ModelView, ModelSQL, ModelStorage, fields
-from trytond.tools import file_open, Cache, safe_eval
-from trytond.backend import TableHandler
-from trytond.pyson import PYSONEncoder, CONTEXT, PYSON
+from __future__ import with_statement
 import base64
 import os
+from trytond.model import ModelView, ModelSQL, ModelStorage, fields
+from trytond.tools import file_open, safe_eval
+from trytond.backend import TableHandler
+from trytond.pyson import PYSONEncoder, CONTEXT, PYSON
+from trytond.transaction import Transaction
+from trytond.cache import Cache
 
 
 class Action(ModelSQL, ModelView):
@@ -28,36 +30,32 @@ class Action(ModelSQL, ModelView):
             'get_action_id': False,
         })
 
-    def default_usage(self, cursor, user, context=None):
+    def default_usage(self):
         return False
 
-    def default_active(self, cursor, user, context=None):
+    def default_active(self):
         return True
 
-    def get_action_id(self, cursor, user, action_id, context=None):
-        if context is None:
-            context = {}
-        ctx = context.copy()
-        ctx['active_test'] = False
-        if self.search(cursor, user, [
-            ('id', '=', action_id),
-            ], context=ctx):
-            return action_id
-        for action_type in (
-                'ir.action.report',
-                'ir.action.act_window',
-                'ir.action.wizard',
-                'ir.action.url',
-                ):
-            action_obj = self.pool.get(action_type)
-            action_id2 = action_obj.search(cursor, user, [
+    def get_action_id(self, action_id):
+        with Transaction().set_context(active_test=False):
+            if self.search([
                 ('id', '=', action_id),
-                ], context=ctx)
-            if action_id2:
-                action = action_obj.browse(cursor, user, action_id2[0],
-                        context=context)
-                return action.action.id
-        return False
+                ]):
+                return action_id
+            for action_type in (
+                    'ir.action.report',
+                    'ir.action.act_window',
+                    'ir.action.wizard',
+                    'ir.action.url',
+                    ):
+                action_obj = self.pool.get(action_type)
+                action_id2 = action_obj.search([
+                    ('id', '=', action_id),
+                    ])
+                if action_id2:
+                    action = action_obj.browse(action_id2[0])
+                    return action.action.id
+            return False
 
 Action()
 
@@ -88,112 +86,104 @@ class ActionKeyword(ModelSQL, ModelView):
             'wrong_wizard_model': 'Wrong wizard model!',
         })
 
-    def check_wizard_model(self, cursor, user, ids):
+    def check_wizard_model(self, ids):
         action_wizard_obj = self.pool.get('ir.action.wizard')
-        for action_keyword in self.browse(cursor, user, ids):
+        for action_keyword in self.browse(ids):
             if action_keyword.action.type == 'ir.action.wizard':
-                action_wizard_id = action_wizard_obj.search(cursor, user, [
+                action_wizard_id = action_wizard_obj.search([
                     ('action', '=', action_keyword.action.id),
                     ], limit=1)[0]
-                action_wizard = action_wizard_obj.browse(cursor, user,
-                        action_wizard_id)
+                action_wizard = action_wizard_obj.browse(action_wizard_id)
                 if action_wizard.model:
                     model, record_id = action_keyword.model.split(',', 1)
                     if model != action_wizard.model:
                         return False
         return True
 
-    def _convert_vals(self, cursor, user, vals, context=None):
+    def _convert_vals(self, vals):
         vals = vals.copy()
         action_obj = self.pool.get('ir.action')
         if 'action' in vals:
-            vals['action'] = action_obj.get_action_id(cursor, user,
-                    vals['action'], context=context)
+            vals['action'] = action_obj.get_action_id(vals['action'])
         return vals
 
-    def models_get(self, cursor, user, context=None):
+    def models_get(self):
         model_obj = self.pool.get('ir.model')
-        model_ids = model_obj.search(cursor, user, [], context=context)
+        model_ids = model_obj.search([])
         res = []
-        for model in model_obj.browse(cursor, user, model_ids,
-                context=context):
+        for model in model_obj.browse(model_ids):
             res.append([model.model, model.name])
         return res
 
-    def delete(self, cursor, user, ids, context=None):
+    def delete(self, ids):
         if isinstance(ids, (int, long)):
             ids = [ids]
-        for keyword in self.browse(cursor, user, ids, context=context):
+        for keyword in self.browse(ids):
             # Restart the cache view
             try:
-                self.pool.get(keyword.model.split(',')[0])\
-                        .fields_view_get(cursor.dbname)
-            except:
+                self.pool.get(keyword.model.split(',')[0]
+                        ).fields_view_get.reset()
+            except Exception:
                 pass
-        return super(ActionKeyword, self).delete(cursor, user, ids,
-                context=context)
+        return super(ActionKeyword, self).delete(ids)
 
-    def create(self, cursor, user, vals, context=None):
-        vals = self._convert_vals(cursor, user, vals, context=context)
+    def create(self, vals):
+        vals = self._convert_vals(vals)
         if 'model' in vals:
             # Restart the cache view
             try:
-                self.pool.get(vals['model'].split(',')[0])\
-                        .fields_view_get(cursor.dbname)
-            except:
+                self.pool.get(vals['model'].split(',')[0]
+                        ).fields_view_get.reset()
+            except Exception:
                 pass
-        return super(ActionKeyword, self).create(cursor, user, vals,
-                context=context)
+        return super(ActionKeyword, self).create(vals)
 
-    def write(self, cursor, user, ids, vals, context=None):
-        vals = self._convert_vals(cursor, user, vals, context=context)
+    def write(self, ids, vals):
+        vals = self._convert_vals(vals)
         if isinstance(ids, (int, long)):
             ids = [ids]
-        for keyword in self.browse(cursor, user, ids, context=context):
+        for keyword in self.browse(ids):
             # Restart the cache view
             try:
-                self.pool.get(keyword.model.split(',')[0])\
-                        .fields_view_get(cursor.dbname)
-            except:
+                self.pool.get(keyword.model.split(',')[0]
+                        ).fields_view_get.reset()
+            except Exception:
                 pass
-        res = super(ActionKeyword, self).write(cursor, user, ids, vals,
-                context=context)
-        for keyword in self.browse(cursor, user, ids, context=context):
+        res = super(ActionKeyword, self).write(ids, vals)
+        for keyword in self.browse(ids):
             # Restart the cache view
             try:
-                self.pool.get(keyword.model.split(',')[0])\
-                        .fields_view_get(cursor.dbname)
-            except:
+                self.pool.get(keyword.model.split(',')[0]
+                        ).fields_view_get.reset()
+            except Exception:
                 pass
         return res
 
-    def get_keyword(self, cursor, user, keyword, value, context=None):
+    def get_keyword(self, keyword, value):
         res = []
         model, model_id = value
 
         action_keyword_ids = []
         if model_id:
-            action_keyword_ids = self.search(cursor, user, [
+            action_keyword_ids = self.search([
                 ('keyword', '=', keyword),
                 ('model', '=', model + ',' + str(model_id)),
-                ], context=context)
-        action_keyword_ids.extend(self.search(cursor, user, [
+                ])
+        action_keyword_ids.extend(self.search([
             ('keyword', '=', keyword),
             ('model', '=', model + ',0'),
-            ], context=context))
+            ]))
         encoder = PYSONEncoder()
-        for action_keyword in self.browse(cursor, user, action_keyword_ids,
-                context=context):
+        for action_keyword in self.browse(action_keyword_ids):
             try:
                 action_obj = self.pool.get(action_keyword.action.type)
-            except:
+            except Exception:
                 continue
-            action_id = action_obj.search(cursor, user, [
+            action_id = action_obj.search([
                 ('action', '=', action_keyword.action.id),
-                ], context=context)
+                ])
             if action_id:
-                res.append(action_obj.read(cursor, user, action_id[0],
-                    context=context))
+                res.append(action_obj.read(action_id[0]))
                 if action_keyword.action.type == 'ir.action.report':
                     del res[-1]['report_content_data']
                     del res[-1]['report_content']
@@ -239,9 +229,10 @@ class ActionReport(ModelSQL, ModelView):
                 'The internal name must be unique by module!'),
         ]
 
-    def init(self, cursor, module_name):
-        super(ActionReport, self).init(cursor, module_name)
+    def init(self, module_name):
+        super(ActionReport, self).init(module_name)
 
+        cursor = Transaction().cursor
         table = TableHandler(cursor, self, module_name)
 
         # Migration from 1.0 report_name_uniq has been removed
@@ -257,61 +248,60 @@ class ActionReport(ModelSQL, ModelView):
                 )
 
             ids = [x[0] for x in cursor.fetchall()]
-            self.write(cursor, 0, ids, {'extension': 'pdf'})
-            ids = self.search(cursor, 0, [('id', 'not in', ids)])
-            self.write(cursor, 0, ids, {'extension': 'odt'})
+            with Transaction().set_user(0):
+                self.write(ids, {'extension': 'pdf'})
+                ids = self.search([('id', 'not in', ids)])
+                self.write(ids, {'extension': 'odt'})
 
             table.drop_column("output_format")
             TableHandler.dropTable(cursor, 'ir.action.report.outputformat',
                       'ir_action_report_outputformat')
 
-    def default_type(self, cursor, user, context=None):
+    def default_type(self):
         return 'ir.action.report'
 
-    def default_report_content(self, cursor, user, context=None):
+    def default_report_content(self):
         return False
 
-    def default_direct_print(self, cursor, user, context=None):
+    def default_direct_print(self):
         return False
 
-    def default_extension(self, cursor, user, context=None):
+    def default_extension(self):
         return 'odt'
 
-    def default_module(self, cursor, user, context=None):
-        return context and context.get('module', '') or ''
+    def default_module(self):
+        return Transaction().context.get('module') or ''
 
-    def get_report_content(self, cursor, user, ids, name, context=None):
+    def get_report_content(self, ids, name):
         res = {}
-        for report in self.browse(cursor, user, ids, context=context):
+        for report in self.browse(ids):
             data = report[name + '_data']
             if not data and report[name[:-8]]:
                 try:
                     data = base64.encodestring(file_open(
                         report[name[:-8]].replace('/', os.sep),
                         mode='rb').read())
-                except:
+                except Exception:
                     data = False
             res[report.id] = data
         return res
 
-    def set_report_content(self, cursor, user, ids, name, value,
-            context=None):
-        self.write(cursor, user, ids, {'%s_data' % name: value},
-                context=context)
+    def set_report_content(self, ids, name, value):
+        self.write(ids, {'%s_data' % name: value})
 
-    def get_style_content(self, cursor, user, ids, name, context=None):
+    def get_style_content(self, ids, name):
         res = {}
-        for report in self.browse(cursor, user, ids, context=context):
+        for report in self.browse(ids):
             try:
                 data = base64.encodestring(file_open(
                     report.style.replace('/', os.sep),
                     mode='rb').read())
-            except:
+            except Exception:
                 data = False
             res[report.id] = data
         return res
 
-    def copy(self, cursor, user, ids, default=None, context=None):
+    def copy(self, ids, default=None):
         if default is None:
             default = {}
 
@@ -321,18 +311,18 @@ class ActionReport(ModelSQL, ModelView):
             ids = [ids]
 
         new_ids = []
-        reports = self.browse(cursor, user, ids, context=context)
+        reports = self.browse(ids)
         for report in reports:
             if report.report:
                 default['report_content'] = False
             default['report_name'] = report.report_name
-            new_ids.append(super(ActionReport, self).copy(cursor, user,
-                report.id, default=default, context=context))
+            new_ids.append(super(ActionReport, self).copy(report.id,
+                default=default))
         if int_id:
             return new_ids[0]
         return new_ids
 
-    def create(self, cursor, user, vals, context=None):
+    def create(self, vals):
         later = {}
         vals = vals.copy()
         for field in vals:
@@ -341,41 +331,36 @@ class ActionReport(ModelSQL, ModelView):
                 later[field] = vals[field]
         for field in later:
             del vals[field]
+        cursor = Transaction().cursor
         if cursor.nextid(self._table):
             cursor.setnextid(self._table, cursor.currid('ir_action'))
-        new_id = super(ActionReport, self).create(cursor, user, vals,
-                context=context)
-        report = self.browse(cursor, user, new_id, context=context)
+        new_id = super(ActionReport, self).create(vals)
+        report = self.browse(new_id)
         new_id = report.action.id
         cursor.execute('UPDATE "' + self._table + '" SET id = %s ' \
                 'WHERE id = %s', (report.action.id, report.id))
         cursor.update_auto_increment(self._table, report.action.id)
-        ModelStorage.delete(self, cursor, user, report.id)
-        self.write(cursor, user, new_id, later, context=context)
+        ModelStorage.delete(self, report.id)
+        self.write(new_id, later)
         return new_id
 
-    def write(self, cursor, user, ids, vals, context=None):
-        if context is None:
-            context = {}
-
+    def write(self, ids, vals):
+        context = Transaction().context
         if 'module' in context:
             vals = vals.copy()
             vals['module'] = context['module']
 
-        return super(ActionReport, self).write(cursor, user, ids, vals,
-                context=context)
+        return super(ActionReport, self).write(ids, vals)
 
-    def delete(self, cursor, user, ids, context=None):
+    def delete(self, ids):
         action_obj = self.pool.get('ir.action')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
-        action_ids = [x.action.id for x in self.browse(cursor, user, ids,
-            context=context)]
+        action_ids = [x.action.id for x in self.browse(ids)]
 
-        res = super(ActionReport, self).delete(cursor, user, ids,
-                context=context)
-        action_obj.delete(cursor, user, action_ids, context=context)
+        res = super(ActionReport, self).delete(ids)
+        action_obj.delete(action_ids)
         return res
 
 ActionReport()
@@ -426,34 +411,34 @@ class ActionActWindow(ModelSQL, ModelView):
             'invalid_search_value': 'Invalid search criteria!',
         })
 
-    def default_type(self, cursor, user, context=None):
+    def default_type(self):
         return 'ir.action.act_window'
 
-    def default_view_type(self, cursor, user, context=None):
+    def default_view_type(self):
         return 'form'
 
-    def default_context(self, cursor, user, context=None):
+    def default_context(self):
         return '{}'
 
-    def default_limit(self, cursor, user, context=None):
+    def default_limit(self):
         return 0
 
-    def default_auto_refresh(self, cursor, user, context=None):
+    def default_auto_refresh(self):
         return 0
 
-    def default_window_name(self, cursor, user, context=None):
+    def default_window_name(self):
         return True
 
-    def default_search_value(self, cursor, user, context=None):
+    def default_search_value(self):
         return '{}'
 
-    def check_domain(self, cursor, user, ids):
+    def check_domain(self, ids):
         "Check domain"
-        for action in self.browse(cursor, user, ids):
+        for action in self.browse(ids):
             if action.domain:
                 try:
                     value = safe_eval(action.domain, CONTEXT)
-                except:
+                except Exception:
                     return False
                 if isinstance(value, PYSON):
                     if not value.types() == set([list]):
@@ -463,17 +448,17 @@ class ActionActWindow(ModelSQL, ModelView):
                 else:
                     try:
                         fields.domain_validate(value)
-                    except:
+                    except Exception:
                         return False
         return True
 
-    def check_context(self, cursor, user, ids):
+    def check_context(self, ids):
         "Check context"
-        for action in self.browse(cursor, user, ids):
+        for action in self.browse(ids):
             if action.context:
                 try:
                     value = safe_eval(action.context, CONTEXT)
-                except:
+                except Exception:
                     return False
                 if isinstance(value, PYSON):
                     if not value.types() == set([dict]):
@@ -483,17 +468,17 @@ class ActionActWindow(ModelSQL, ModelView):
                 else:
                     try:
                         fields.context_validate(value)
-                    except:
+                    except Exception:
                         return False
         return True
 
-    def check_search_value(self, cursor, user, ids):
+    def check_search_value(self, ids):
         "Check search_value"
-        for action in self.browse(cursor, user, ids):
+        for action in self.browse(ids):
             if action.search_value:
                 try:
                     value = safe_eval(action.search_value, CONTEXT)
-                except:
+                except Exception:
                     return False
                 if isinstance(value, PYSON):
                     if not value.types() == set([dict]):
@@ -502,14 +487,14 @@ class ActionActWindow(ModelSQL, ModelView):
                     return False
         return True
 
-    def get_views(self, cursor, user, ids, name, context=None):
+    def get_views(self, ids, name):
         res = {}
-        for act in self.browse(cursor, user, ids, context=context):
-            res[act.id] = [(view.view.id, view.view.type) \
+        for act in self.browse(ids):
+            res[act.id] = [(view.view.id, view.view.type)
                     for view in act.act_window_views]
         return res
 
-    def get_pyson(self, cursor, user, ids, name, context=None):
+    def get_pyson(self, ids, name):
         res = {}
         encoder = PYSONEncoder()
         field = name[6:]
@@ -518,12 +503,12 @@ class ActionActWindow(ModelSQL, ModelView):
             'context': '{}',
             'search_value': '{}',
         }
-        for act in self.browse(cursor, user, ids, context=context):
+        for act in self.browse(ids):
             res[act.id] = encoder.encode(safe_eval(act[field] or
                 defaults.get(field, 'False'), CONTEXT))
         return res
 
-    def create(self, cursor, user, vals, context=None):
+    def create(self, vals):
         later = {}
         vals = vals.copy()
         for field in vals:
@@ -532,30 +517,28 @@ class ActionActWindow(ModelSQL, ModelView):
                 later[field] = vals[field]
         for field in later:
             del vals[field]
+        cursor = Transaction().cursor
         if cursor.nextid(self._table):
             cursor.setnextid(self._table, cursor.currid('ir_action'))
-        new_id = super(ActionActWindow, self).create(cursor, user, vals,
-                context=context)
-        act_window = self.browse(cursor, user, new_id, context=context)
+        new_id = super(ActionActWindow, self).create(vals)
+        act_window = self.browse(new_id)
         new_id = act_window.action.id
         cursor.execute('UPDATE "' + self._table + '" SET id = %s ' \
                 'WHERE id = %s', (act_window.action.id, act_window.id))
         cursor.update_auto_increment(self._table, act_window.action.id)
-        ModelStorage.delete(self, cursor, user, act_window.id)
-        self.write(cursor, user, new_id, later, context=context)
+        ModelStorage.delete(self, act_window.id)
+        self.write(new_id, later)
         return new_id
 
-    def delete(self, cursor, user, ids, context=None):
+    def delete(self, ids):
         action_obj = self.pool.get('ir.action')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
-        action_ids = [x.action.id for x in self.browse(cursor, user, ids,
-            context=context)]
+        action_ids = [x.action.id for x in self.browse(ids)]
 
-        res = super(ActionActWindow, self).delete(cursor, user, ids,
-                context=context)
-        action_obj.delete(cursor, user, action_ids, context=context)
+        res = super(ActionActWindow, self).delete(ids)
+        action_obj.delete(action_ids)
         return res
 
 ActionActWindow()
@@ -576,9 +559,9 @@ class ActionActWindowView(ModelSQL, ModelView):
         super(ActionActWindowView, self).__init__()
         self._order.insert(0, ('sequence', 'ASC'))
 
-    def init(self, cursor, module_name):
-        super(ActionActWindowView, self).init(cursor, module_name)
-        table = TableHandler(cursor, self, module_name)
+    def init(self, module_name):
+        super(ActionActWindowView, self).init(module_name)
+        table = TableHandler(Transaction().cursor, self, module_name)
 
         # Migration from 1.0 remove multi
         table.drop_column('multi')
@@ -598,10 +581,10 @@ class ActionWizard(ModelSQL, ModelView):
     email = fields.Char('Email')
     window = fields.Boolean('Window', help='Run wizard in a new window')
 
-    def default_type(self, cursor, user, context=None):
+    def default_type(self):
         return 'ir.action.wizard'
 
-    def create(self, cursor, user, vals, context=None):
+    def create(self, vals):
         later = {}
         vals = vals.copy()
         for field in vals:
@@ -610,30 +593,28 @@ class ActionWizard(ModelSQL, ModelView):
                 later[field] = vals[field]
         for field in later:
             del vals[field]
+        cursor = Transaction().cursor
         if cursor.nextid(self._table):
             cursor.setnextid(self._table, cursor.currid('ir_action'))
-        new_id = super(ActionWizard, self).create(cursor, user, vals,
-                context=context)
-        wizard = self.browse(cursor, user, new_id, context=context)
+        new_id = super(ActionWizard, self).create(vals)
+        wizard = self.browse(new_id)
         new_id = wizard.action.id
         cursor.execute('UPDATE "' + self._table + '" SET id = %s ' \
                 'WHERE id = %s', (wizard.action.id, wizard.id))
         cursor.update_auto_increment(self._table, wizard.action.id)
-        ModelStorage.delete(self, cursor, user, wizard.id)
-        self.write(cursor, user, new_id, later, context=context)
+        ModelStorage.delete(self, wizard.id)
+        self.write(new_id, later)
         return new_id
 
-    def delete(self, cursor, user, ids, context=None):
+    def delete(self, ids):
         action_obj = self.pool.get('ir.action')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
-        action_ids = [x.action.id for x in self.browse(cursor, user, ids,
-            context=context)]
+        action_ids = [x.action.id for x in self.browse(ids)]
 
-        res = super(ActionWizard, self).delete(cursor, user, ids,
-                context=context)
-        action_obj.delete(cursor, user, action_ids, context=context)
+        res = super(ActionWizard, self).delete(ids)
+        action_obj.delete(action_ids)
         return res
 
 ActionWizard()
@@ -656,76 +637,66 @@ class ActionWizardSize(ModelSQL, ModelView):
             'set_size': True,
         })
 
-    def create(self, cursor, user, vals, context=None):
-        res = super(ActionWizardSize, self).create(cursor, user, vals,
-                context=context)
+    def create(self, vals):
+        res = super(ActionWizardSize, self).create(vals)
         # Restart the cache for get_size
-        self.get_size(cursor.dbname)
+        self.get_size.reset()
         return res
 
-    def write(self, cursor, user, ids, vals, context=None):
-        res = super(ActionWizardSize, self).write(cursor, user, ids,
-                vals, context=context)
+    def write(self, ids, vals):
+        res = super(ActionWizardSize, self).write(ids, vals)
         # Restart the cache for get_size
-        self.get_size(cursor.dbname)
+        self.get_size.reset()
         return res
 
-    def delete(self, cursor, user, ids, context=None):
-        res = super(ActionWizardSize, self).delete(cursor, user, ids,
-                context=context)
+    def delete(self, ids):
+        res = super(ActionWizardSize, self).delete(ids)
         # Restart the cache for get_size
-        self.get_size(cursor.dbname)
+        self.get_size.reset()
         return res
 
-    def set_size(self, cursor, user, wizard, model, width, height,
-            context=None):
+    def set_size(self, wizard, model, width, height):
         '''
         Set size for wizard dialog.
-        :param cursor: the database cursor
-        :param user: the user id
         :param wizard: the wizard name
         :param model: the model name
         :param width: the width
         :param height: the height
-        :param context: the context
         '''
-        ids = self.search(cursor, user, [
-            ('user', '=', user),
+        ids = self.search([
+            ('user', '=', Transaction().user),
             ('wizard', '=', wizard),
             ('model', '=', model),
-            ], context=context)
+            ])
         if ids:
-            self.write(cursor, user, ids, {
+            self.write(ids, {
                 'width': width,
                 'height': height,
-                }, context=context)
+                })
         else:
-            self.create(cursor, user, {
+            self.create({
                 'wizard': wizard,
                 'model': model,
-                'user': user,
+                'user': Transaction().user,
                 'width': width,
                 'height': height,
-                }, context=context)
+                })
 
     @Cache('ir.action.wizard_size.get_size')
-    def get_size(self, cursor, user, wizard, model, context=None):
+    def get_size(self, wizard, model):
         '''
         Get size for wizard dialog.
-        :param cursor: the database cursor
-        :param user: the user id
         :param wizard: the wizard name
         :param model: the model name
-        :param context: the context
         :return: (width, height)
         '''
-        ids = self.search(cursor, user, [
-            ('user', '=', user),
+        ids = self.search([
+            ('user', '=', Transaction().user),
             ('wizard', '=', wizard),
             ('model', '=', model),
-            ], limit=1, context=context)
+            ], limit=1)
         if ids:
-            wizard_size = self.browse(cursor, user, ids[0], context=context)
+            wizard_size = self.browse(ids[0])
             return wizard_size.width, wizard_size.height
         return (0, 0)
 
@@ -741,13 +712,13 @@ class ActionURL(ModelSQL, ModelView):
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
 
-    def default_type(self, cursor, user, context=None):
+    def default_type(self):
         return 'ir.action.url'
 
-    def default_target(self, cursor, user, context=None):
+    def default_target(self):
         return 'new'
 
-    def create(self, cursor, user, vals, context=None):
+    def create(self, vals):
         later = {}
         vals = vals.copy()
         for field in vals:
@@ -756,30 +727,28 @@ class ActionURL(ModelSQL, ModelView):
                 later[field] = vals[field]
         for field in later:
             del vals[field]
+        cursor = Transaction().cursor
         if cursor.nextid(self._table):
             cursor.setnextid(self._table, cursor.currid('ir_action'))
-        new_id = super(ActionURL, self).create(cursor, user, vals,
-                context=context)
-        url = self.browse(cursor, user, new_id, context=context)
+        new_id = super(ActionURL, self).create(vals)
+        url = self.browse(new_id)
         new_id = url.action.id
         cursor.execute('UPDATE "' + self._table + '" SET id = %s ' \
                 'WHERE id = %s', (url.action.id, url.id))
         cursor.update_auto_increment(self._table, url.action.id)
-        ModelStorage.delete(self, cursor, user, url.id)
-        self.write(cursor, user, new_id, later, context=context)
+        ModelStorage.delete(self, url.id)
+        self.write(new_id, later)
         return new_id
 
-    def delete(self, cursor, user, ids, context=None):
+    def delete(self, ids):
         action_obj = self.pool.get('ir.action')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
-        action_ids = [x.action.id for x in self.browse(cursor, user, ids,
-            context=context)]
+        action_ids = [x.action.id for x in self.browse(ids)]
 
-        res = super(ActionURL, self).delete(cursor, user, ids,
-                context=context)
-        action_obj.delete(cursor, user, action_ids, context=context)
+        res = super(ActionURL, self).delete(ids)
+        action_obj.delete(action_ids)
         return res
 
 ActionURL()
