@@ -11,17 +11,14 @@ class Binary(Field):
     _type = 'binary'
 
     @staticmethod
-    def get(cursor, user, ids, model, name, values=None, context=None):
+    def get(ids, model, name, values=None):
         '''
         Convert the binary value into ``str``
 
-        :param cursor: the database cursor
-        :param user: the user id
         :param ids: a list of ids
         :param model: a string with the name of the model
         :param name: a string with the name of the field
         :param values: a dictionary with the read values
-        :param context: the context
         :return: a dictionary with ids as key and values as value
         '''
         if values is None:
