@@ -1,8 +1,8 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-
-from trytond.model.fields.field import Field
 from itertools import chain
+from trytond.model.fields.field import Field
+from trytond.transaction import Transaction
 
 
 class Many2Many(Field):
@@ -45,17 +45,14 @@ class Many2Many(Field):
         self.datetime_field = datetime_field
     __init__.__doc__ += Field.__init__.__doc__
 
-    def get(self, cursor, user, ids, model, name, values=None, context=None):
+    def get(self, ids, model, name, values=None):
         '''
         Return target records ordered.
 
-        :param cursor: the database cursor
-        :param user: the user id
         :param ids: a list of ids
         :param model: a string with the name of the model
         :param name: a string with the name of the field
         :param values: a dictionary with the read values
-        :param context: the context
         :return: a dictionary with ids as key and values as value
         '''
         if values is None:
@@ -69,25 +66,23 @@ class Many2Many(Field):
         relation_obj = model.pool.get(self.relation_name)
 
         relation_ids = []
-        for i in range(0, len(ids), cursor.IN_MAX):
-            sub_ids = ids[i:i + cursor.IN_MAX]
-            relation_ids.append(relation_obj.search(cursor, user, [
+        for i in range(0, len(ids), Transaction().cursor.IN_MAX):
+            sub_ids = ids[i:i + Transaction().cursor.IN_MAX]
+            relation_ids.append(relation_obj.search([
                 (self.origin, 'in', sub_ids),
                 (self.target + '.id', '!=', False),
-                ], order=self.order, context=context))
+                ], order=self.order))
         relation_ids = list(chain(*relation_ids))
 
-        for relation in relation_obj.read(cursor, user, relation_ids,
-                [self.origin, self.target], context=context):
+        for relation in relation_obj.read(relation_ids,
+                [self.origin, self.target]):
             res[relation[self.origin]].append(relation[self.target])
         return res
 
-    def set(self, cursor, user, ids, model, name, values, context=None):
+    def set(self, ids, model, name, values):
         '''
         Set the values.
 
-        :param cursor: The database cursor
-        :param user: The user id
         :param ids: A list of ids
         :param model: A string with the name of the model
         :param name: A string with the name of the field
@@ -99,7 +94,6 @@ class Many2Many(Field):
             (``add``, ``<ids>``),
             (``unlink_all``),
             (``set``, ``<ids>``)
-        :param context: The context
         '''
         if not values:
             return
@@ -108,15 +102,14 @@ class Many2Many(Field):
         for act in values:
             if act[0] == 'create':
                 for record_id in ids:
-                    relation_obj.create(cursor, user, {
+                    relation_obj.create({
                         self.origin: record_id,
-                        self.target: target_obj.create(cursor, user, act[1],
-                            context=context),
-                        }, context=context)
+                        self.target: target_obj.create(act[1]),
+                        })
             elif act[0] == 'write':
-                target_obj.write(cursor, user, act[1] , act[2], context=context)
+                target_obj.write(act[1] , act[2])
             elif act[0] == 'delete':
-                target_obj.delete(cursor, user, act[1], context=context)
+                target_obj.delete(act[1])
             elif act[0] == 'unlink':
                 if isinstance(act[1], (int, long)):
                     target_ids = [act[1]]
@@ -125,13 +118,13 @@ class Many2Many(Field):
                 if not target_ids:
                     continue
                 relation_ids = []
-                for i in range(0, len(target_ids), cursor.IN_MAX):
-                    sub_ids = target_ids[i:i + cursor.IN_MAX]
-                    relation_ids += relation_obj.search(cursor, user, [
+                for i in range(0, len(target_ids), Transaction().cursor.IN_MAX):
+                    sub_ids = target_ids[i:i + Transaction().cursor.IN_MAX]
+                    relation_ids += relation_obj.search([
                         (self.origin, 'in', ids),
                         (self.target, 'in', sub_ids),
-                        ], context=context)
-                relation_obj.delete(cursor, user, relation_ids, context=context)
+                        ])
+                relation_obj.delete(relation_ids)
             elif act[0] == 'add':
                 if isinstance(act[1], (int, long)):
                     target_ids = [act[1]]
@@ -140,44 +133,43 @@ class Many2Many(Field):
                 if not target_ids:
                     continue
                 existing_ids = []
-                for i in range(0, len(target_ids), cursor.IN_MAX):
-                    sub_ids = target_ids[i:i + cursor.IN_MAX]
-                    relation_ids = relation_obj.search(cursor, user, [
+                for i in range(0, len(target_ids), Transaction().cursor.IN_MAX):
+                    sub_ids = target_ids[i:i + Transaction().cursor.IN_MAX]
+                    relation_ids = relation_obj.search([
                         (self.origin, 'in', ids),
                         (self.target, 'in', sub_ids),
-                        ], context=context)
-                    for relation in relation_obj.browse(cursor, user,
-                            relation_ids, context=context):
+                        ])
+                    for relation in relation_obj.browse(relation_ids):
                         existing_ids.append(relation[self.target].id)
                 for new_id in (x for x in target_ids if x not in existing_ids):
                     for record_id in ids:
-                        relation_obj.create(cursor, user, {
+                        relation_obj.create({
                             self.origin: record_id,
                             self.target: new_id,
-                            }, context=context)
+                            })
             elif act[0] == 'unlink_all':
-                target_ids = relation_obj.search(cursor, user, [
+                target_ids = relation_obj.search([
                     (self.origin, 'in', ids),
                     (self.target + '.id', '!=', False),
-                    ], context=context)
-                relation_obj.delete(cursor, user, target_ids, context=context)
+                    ])
+                relation_obj.delete(target_ids)
             elif act[0] == 'set':
                 if not act[1]:
                     target_ids = []
                 else:
                     target_ids = list(act[1])
-                target_ids2 = relation_obj.search(cursor, user, [
+                target_ids2 = relation_obj.search([
                     (self.origin, 'in', ids),
                     (self.target + '.id', '!=', False),
-                    ], context=context)
-                relation_obj.delete(cursor, user, target_ids2, context=context)
+                    ])
+                relation_obj.delete(target_ids2)
 
                 for new_id in target_ids:
                     for record_id in ids:
-                        relation_obj.create(cursor, user, {
+                        relation_obj.create({
                             self.origin: record_id,
                             self.target: new_id,
-                            }, context=context)
+                            })
             else:
                 raise Exception('Bad arguments')
 
