@@ -4,6 +4,7 @@ from trytond.pysocket import PySocket
 from trytond.protocols.sslsocket import SSLSocket
 from trytond.config import CONFIG
 from trytond.protocols.dispatcher import dispatch
+from trytond.protocols.common import endsocket
 import threading
 import select
 import traceback
@@ -13,7 +14,7 @@ import sys
 
 class NetRPCClientThread(threading.Thread):
     def __init__(self, sock, threads, secure):
-        threading.Thread.__init__(self)
+        threading.Thread.__init__(self, name='NetRPCClientThread')
         self.sock = sock
         self.threads = threads
         self.running = False
@@ -23,13 +24,16 @@ class NetRPCClientThread(threading.Thread):
         self.running = True
         try:
             pysocket = PySocket(self.sock)
-        except:
+        except Exception:
             self.sock.close()
             self.threads.remove(self)
             return False
         timeout = 0
         while self.running:
-            (rlist, _, _) = select.select([self.sock], [], [], 1)
+            try:
+                rlist, _, _ = select.select([self.sock], [], [], 1)
+            except select.error:
+                continue
             if not rlist:
                 timeout += 1
                 if timeout > 600:
@@ -38,7 +42,7 @@ class NetRPCClientThread(threading.Thread):
             timeout = 0
             try:
                 msg = pysocket.receive()
-            except:
+            except Exception:
                 pysocket.disconnect()
                 self.threads.remove(self)
                 return False
@@ -51,7 +55,7 @@ class NetRPCClientThread(threading.Thread):
                 for line in traceback.format_exception(*sys.exc_info()):
                     try:
                         line = line.encode('utf-8', 'ignore')
-                    except:
+                    except Exception:
                         continue
                     tb_s += line
                 for path in sys.path:
@@ -63,7 +67,7 @@ class NetRPCClientThread(threading.Thread):
                 try:
                     pysocket.send(exception.args, exception=True,
                             traceback=tb_s)
-                except:
+                except Exception:
                     pysocket.disconnect()
                     self.threads.remove(self)
                     return False
@@ -77,13 +81,13 @@ class NetRPCClientThread(threading.Thread):
 
 class NetRPCServerThread(threading.Thread):
     def __init__(self, interface, port, secure=False):
-        threading.Thread.__init__(self)
+        threading.Thread.__init__(self, name='NetRPCServerThread')
         self.socket = None
         if socket.has_ipv6:
             try:
                 socket.getaddrinfo(interface or None, port, socket.AF_INET6)
                 self.socket = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
-            except:
+            except Exception:
                 pass
         if self.socket is None:
             self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -102,15 +106,21 @@ class NetRPCServerThread(threading.Thread):
             while self.running:
                 if not int(CONFIG['max_thread']) \
                         or len(self.threads) < int(CONFIG['max_thread']):
+                    try:
+                        rlist, _, _ = select.select([self.socket], [], [], 1)
+                    except select.error:
+                        continue
+                    if not rlist:
+                        continue
                     (clientsocket, _) = self.socket.accept()
                     c_thread = NetRPCClientThread(clientsocket, self.threads,
                             self.secure)
                     self.threads.append(c_thread)
                     c_thread.start()
-        except:
+        except Exception:
             try:
                 self.socket.close()
-            except:
+            except Exception:
                 pass
             return False
 
@@ -121,13 +131,9 @@ class NetRPCServerThread(threading.Thread):
                 thread = self.threads[0]
                 thread.stop()
                 thread.join()
-            except:
+            except Exception:
                 pass
         try:
-            if hasattr(socket, 'SHUT_RDWR'):
-                self.socket.shutdown(socket.SHUT_RDWR)
-            else:
-                self.socket.shutdown(2)
-            self.socket.close()
-        except:
+            endsocket(self.socket)
+        except Exception:
             return False
