@@ -4,11 +4,11 @@ from trytond.protocols.sslsocket import SSLSocket
 from trytond.protocols.dispatcher import dispatch
 from trytond.config import CONFIG
 from trytond.protocols.datatype import Float
+from trytond.protocols.common import daemon
 from trytond import security
 import SimpleXMLRPCServer
 import SocketServer
 import xmlrpclib
-import threading
 import traceback
 import socket
 import sys
@@ -17,17 +17,36 @@ import gzip
 import StringIO
 import base64
 import datetime
+from types import DictType
 
 # convert decimal to float before marshalling:
 from decimal import Decimal
-xmlrpclib.Marshaller.dispatch[Decimal] = \
-        lambda self, value, write: self.dump_double(float(value), write)
+
+def dump_decimal(self, value, write):
+    write("<value><double>")
+    write(str(value))
+    write("</double></value>\n")
+
+xmlrpclib.Marshaller.dispatch[Decimal] = dump_decimal
 xmlrpclib.Marshaller.dispatch[type(None)] = \
         lambda self, value, write: self.dump_bool(bool(value), write)
 xmlrpclib.Marshaller.dispatch[datetime.date] = \
         lambda self, value, write: self.dump_datetime(
                 datetime.datetime.combine(value, datetime.time()), write)
 
+def dump_struct(self, value, write, escape=xmlrpclib.escape):
+    converted_value = {}
+    for k, v in value.items():
+        if type(k) in (int, long):
+            k = str(int(k))
+        elif type(k) == float:
+            k = repr(k)
+        converted_value[k] = v
+    return self.dump_struct(converted_value, write, escape=escape)
+
+xmlrpclib.Marshaller.dispatch[DictType] = dump_struct
+
+
 def _end_double(self, data):
     self.append(Float(data))
     self._value = 0
@@ -63,12 +82,12 @@ class GenericXMLRPCRequestHandler:
                     }
                 return dispatch(host, port, 'XML-RPC', database_name, user,
                         session, object_type, object_name, method, *params)
-            except:
+            except Exception:
                 tb_s = ''
                 for line in traceback.format_exception(*sys.exc_info()):
                     try:
                         line = line.encode('utf-8', 'ignore')
-                    except:
+                    except Exception:
                         continue
                     tb_s += line
                 for path in sys.path:
@@ -101,7 +120,7 @@ class SimpleXMLRPCRequestHandler(GenericXMLRPCRequestHandler,
                         password)
                 self.tryton = {'user': user_id, 'session': session}
                 return res
-        except:
+        except Exception:
             pass
         self.send_error(401, 'Unauthorized')
         self.send_header("WWW-Authenticate", 'Basic realm="Tryton"')
@@ -142,7 +161,7 @@ class SimpleXMLRPCRequestHandler(GenericXMLRPCRequestHandler,
             response = self.server._marshaled_dispatch(
                     data, getattr(self, '_dispatch', None)
                 )
-        except: # This should only happen if the module is buggy
+        except Exception: # This should only happen if the module is buggy
             # internal error, report as HTTP server error
             self.send_response(500)
             self.end_headers()
@@ -184,6 +203,8 @@ class SimpleThreadedXMLRPCServer(SocketServer.ThreadingMixIn,
     timeout = 1
 
     def server_bind(self):
+        # Python < 2.6 doesn't handle self.timeout
+        self.socket.settimeout(1)
         self.socket.setsockopt(socket.SOL_SOCKET,
                 socket.SO_REUSEADDR, 1)
         SimpleXMLRPCServer.SimpleXMLRPCServer.server_bind(self)
@@ -208,42 +229,18 @@ class SecureThreadedXMLRPCServer6(SecureThreadedXMLRPCServer):
     address_family = socket.AF_INET6
 
 
-class XMLRPCDaemon(threading.Thread):
+class XMLRPCDaemon(daemon):
 
     def __init__(self, interface, port, secure=False):
-        threading.Thread.__init__(self)
-        self.secure = secure
-        self.running = False
-        ipv6 = False
-        if socket.has_ipv6:
-            try:
-                socket.getaddrinfo(interface or None, port, socket.AF_INET6)
-                ipv6 = True
-            except:
-                pass
-        if secure:
+        daemon.__init__(self, interface, port, secure, name='XMLRPCDaemon')
+        if self.secure:
             handler_class = SecureXMLRPCRequestHandler
             server_class = SecureThreadedXMLRPCServer
-            if ipv6:
+            if self.ipv6:
                 server_class = SecureThreadedXMLRPCServer6
         else:
             handler_class = SimpleXMLRPCRequestHandler
             server_class = SimpleThreadedXMLRPCServer
-            if ipv6:
+            if self.ipv6:
                 server_class = SimpleThreadedXMLRPCServer6
         self.server = server_class((interface, port), handler_class, 0)
-
-    def stop(self):
-        self.running = False
-        if os.name != 'nt':
-            if hasattr(socket, 'SHUT_RDWR'):
-                self.server.socket.shutdown(socket.SHUT_RDWR)
-            else:
-                self.server.socket.shutdown(2)
-        self.server.socket.close()
-
-    def run(self):
-        self.running = True
-        while self.running:
-            self.server.handle_request()
-        return True
