@@ -2,6 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.backend import TableHandler
+from trytond.transaction import Transaction
 
 
 class UIMenuGroup(ModelSQL):
@@ -13,12 +14,13 @@ class UIMenuGroup(ModelSQL):
     gid = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=1, required=True)
 
-    def init(self, cursor, module_name):
+    def init(self, module_name):
+        cursor = Transaction().cursor
         # Migration from 1.0 table name change
         TableHandler.table_rename(cursor, 'ir_ui_menu_group_rel', self._table)
         TableHandler.sequence_rename(cursor, 'ir_ui_menu_group_rel_id_seq',
                 self._table + '_id_seq')
-        super(UIMenuGroup, self).init(cursor, module_name)
+        super(UIMenuGroup, self).init(module_name)
 
 UIMenuGroup()
 
@@ -32,12 +34,13 @@ class ActionGroup(ModelSQL):
     gid = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=1, required=True)
 
-    def init(self, cursor, module_name):
+    def init(self, module_name):
+        cursor = Transaction().cursor
         # Migration from 1.0 table name change
         TableHandler.table_rename(cursor, 'ir_action_group_rel', self._table)
         TableHandler.sequence_rename(cursor, 'ir_action_group_rel_id_seq',
                 self._table + '_id_seq')
-        super(ActionGroup, self).init(cursor, module_name)
+        super(ActionGroup, self).init(module_name)
 
 ActionGroup()
 
@@ -51,12 +54,13 @@ class ModelFieldGroup(ModelSQL):
     group_id = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=1, required=True)
 
-    def init(self, cursor, module_name):
+    def init(self, module_name):
+        cursor = Transaction().cursor
         # Migration from 1.0 table name change
         TableHandler.table_rename(cursor, 'ir_model_field_group_rel', self._table)
         TableHandler.sequence_rename(cursor, 'ir_model_field_group_rel_id_seq',
                 self._table + '_id_seq')
-        super(ModelFieldGroup, self).init(cursor, module_name)
+        super(ModelFieldGroup, self).init(module_name)
 
 ModelFieldGroup()
 
@@ -70,12 +74,13 @@ class RuleGroupGroup(ModelSQL):
     group_id = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=1, required=True)
 
-    def init(self, cursor, module_name):
+    def init(self, module_name):
+        cursor = Transaction().cursor
         # Migration from 1.0 table name change
         TableHandler.table_rename(cursor, 'group_rule_group_rel', self._table)
         TableHandler.sequence_rename(cursor, 'group_rule_group_rel_id_seq',
                 self._table + '_id_seq')
-        super(RuleGroupGroup, self).init(cursor, module_name)
+        super(RuleGroupGroup, self).init(module_name)
 
 RuleGroupGroup()
 
@@ -89,12 +94,13 @@ class RuleGroupUser(ModelSQL):
     user_id = fields.Many2One('res.user', 'User', ondelete='CASCADE',
             select=1, required=True)
 
-    def init(self, cursor, module_name):
+    def init(self, module_name):
+        cursor = Transaction().cursor
         # Migration from 1.0 table name change
         TableHandler.table_rename(cursor, 'user_rule_group_rel', self._table)
         TableHandler.sequence_rename(cursor, 'user_rule_group_rel_id_seq',
                 self._table + '_id_seq')
-        super(RuleGroupUser, self).init(cursor, module_name)
+        super(RuleGroupUser, self).init(module_name)
 
 RuleGroupUser()
 
@@ -102,10 +108,10 @@ RuleGroupUser()
 class Lang(ModelSQL, ModelView):
     _name = 'ir.lang'
 
-    def write(self, cursor, user, ids, vals, context=None):
-        res = super(Lang, self).write(cursor, user, ids, vals, context=context)
+    def write(self, ids, vals):
+        res = super(Lang, self).write(ids, vals)
         # Restart the cache for get_preferences
-        self.pool.get('res.user').get_preferences(cursor.dbname)
+        self.pool.get('res.user').get_preferences.reset()
         return res
 
 Lang()
@@ -129,28 +135,25 @@ class SequenceTypeGroup(ModelSQL):
     group = fields.Many2One('res.group', 'User Groups',
             ondelete='CASCADE', select=1, required=True)
 
-    def delete(self, cursor, user, ids, context=None):
+    def delete(self, ids):
         rule_obj = self.pool.get('ir.rule')
-        res = super(SequenceTypeGroup, self).delete(cursor, user, ids,
-                context=context)
+        res = super(SequenceTypeGroup, self).delete(ids)
         # Restart the cache on the domain_get method of ir.rule
-        rule_obj.domain_get(cursor.dbname)
+        rule_obj.domain_get.reset()
         return res
 
-    def create(self, cursor, user, vals, context=None):
+    def create(self, vals):
         rule_obj = self.pool.get('ir.rule')
-        res = super(SequenceTypeGroup, self).create(cursor, user, vals,
-                context=context)
+        res = super(SequenceTypeGroup, self).create(vals)
         # Restart the cache on the domain_get method of ir.rule
-        rule_obj.domain_get(cursor.dbname)
+        rule_obj.domain_get.reset()
         return res
 
-    def write(self, cursor, user, ids, vals, context=None):
+    def write(self, ids, vals):
         rule_obj = self.pool.get('ir.rule')
-        res = super(SequenceTypeGroup, self).write(cursor, user, ids, vals,
-                context=context)
+        res = super(SequenceTypeGroup, self).write(ids, vals)
         # Restart the cache on the domain_get method
-        rule_obj.domain_get(cursor.dbname)
+        rule_obj.domain_get.reset()
         return res
 
 SequenceTypeGroup()
@@ -161,18 +164,17 @@ class Sequence(ModelSQL, ModelView):
     groups = fields.Function(fields.Many2Many('res.group', None, None,
         'User Groups'), 'get_groups', searcher='search_groups')
 
-    def get_groups(self, cursor, user, ids, name, context=None):
+    def get_groups(self, ids, name):
         sequence_type_obj = self.pool.get('ir.sequence.type')
-        sequences= self.browse(cursor, user, ids, context=context)
+        sequences= self.browse(ids)
         code2seq = {}
         for sequence in sequences:
             code2seq.setdefault(sequence.code, []).append(sequence.id)
 
-        sequence_type_ids = sequence_type_obj.search(cursor, user, [
+        sequence_type_ids = sequence_type_obj.search([
                 ('code', 'in', code2seq.keys()),
-                ], context=context)
-        sequence_types = sequence_type_obj.browse(cursor, user,
-                sequence_type_ids, context=context)
+                ])
+        sequence_types = sequence_type_obj.browse(sequence_type_ids)
 
         res = {}
         for sequence_type in sequence_types:
@@ -182,12 +184,10 @@ class Sequence(ModelSQL, ModelView):
 
         return res
 
-    def search_groups(self, cursor, user, name, clause, context=None):
+    def search_groups(self, name, clause):
         sequence_type_obj = self.pool.get('ir.sequence.type')
-        ids = sequence_type_obj.search(cursor, user, [clause], order=[],
-                context=context)
-        seq_types = sequence_type_obj.browse(cursor, user, ids,
-                context=context)
+        ids = sequence_type_obj.search([clause], order=[])
+        seq_types = sequence_type_obj.browse(ids)
         codes = set(st.code for st in seq_types)
         return [('code', 'in', list(codes))]
 
