@@ -4,21 +4,19 @@
 """
 Miscelleanous tools used by tryton
 """
-import os, time, sys
-import inspect
-from trytond.config import CONFIG
+import os, sys
 import subprocess
 import zipfile
-from trytond.backend import Database
-from threading import Lock, local
+from threading import local
 import smtplib
 try:
     import cStringIO as StringIO
 except ImportError:
     import StringIO
 import dis
-import datetime
 from decimal import Decimal
+from trytond.config import CONFIG
+from trytond.const import OPERATORS
 
 def find_in_path(name):
     if os.name == "nt":
@@ -133,7 +131,7 @@ def file_open(name, mode="r", subdir='modules'):
                 return StringIO.StringIO(zfile.read(os.path.join(
                     os.path.basename(head), zipname).replace(
                         os.sep, '/')))
-            except:
+            except Exception:
                 name2 = os.path.normpath(os.path.join(head + '.zip', zipname))
     for i in (name2, name, name3):
         if i and os.path.isfile(i):
@@ -171,167 +169,6 @@ def find_language_context(args, kargs=None):
             res = arg.get('language', 'en_US')
     return kargs.get('context', {}).get('language', res)
 
-
-class Cache(object):
-    """
-    Use it as a decorator of the function you plan to cache
-    Timeout: 0 = no timeout, otherwise in seconds
-    """
-    _cache_instance = []
-    _resets = {}
-    _resets_lock = Lock()
-
-    def __init__(self, name, timeout=3600, max_len=1024):
-        self.timeout = timeout
-        self.max_len = max_len
-        self._cache = {}
-        self._cache_instance.append(self)
-        self._name = name
-        self._timestamp = None
-        self._lock = Lock()
-
-    def __call__(self, function):
-        arg_names = inspect.getargspec(function)[0][2:]
-
-        def cached_result(self2, cursor=None, *args, **kwargs):
-            result = None
-            find = False
-            if isinstance(cursor, basestring):
-                Cache.reset(cursor, self._name)
-                self._lock.acquire()
-                try:
-                    self._cache[cursor] = {}
-                finally:
-                    self._lock.release()
-                return True
-            # Update named arguments with positional argument values
-            kwargs_origin = kwargs.copy()
-            kwargs.update(dict(zip(arg_names, args)))
-            if 'context' in kwargs:
-                if isinstance(kwargs['context'], dict):
-                    kwargs['context'] = kwargs['context'].copy()
-                    for i in ('_timestamp', '_delete', '_create_records',
-                            '_delete_records'):
-                        if i in kwargs['context']:
-                            del kwargs['context'][i]
-            kwargs = kwargs.items()
-            kwargs.sort()
-
-            self._lock.acquire()
-            try:
-                self._cache.setdefault(cursor.dbname, {})
-            finally:
-                self._lock.release()
-
-            lower = None
-            self._lock.acquire()
-            try:
-                if len(self._cache[cursor.dbname]) > self.max_len:
-                    mintime = time.time() - self.timeout
-                    for key in self._cache[cursor.dbname].keys():
-                        last_time = self._cache[cursor.dbname][key][1]
-                        if mintime > last_time:
-                            del self._cache[cursor.dbname][key]
-                        else:
-                            if not lower or lower[1] > last_time:
-                                lower = (key, last_time)
-                if len(self._cache[cursor.dbname]) > self.max_len and lower:
-                    del self._cache[cursor.dbname][lower[0]]
-            finally:
-                self._lock.release()
-
-            # Work out key as a tuple
-            key = (id(self2), repr(kwargs))
-
-            # Check cache and return cached value if possible
-            self._lock.acquire()
-            try:
-                if key in self._cache[cursor.dbname]:
-                    (value, last_time) = self._cache[cursor.dbname][key]
-                    mintime = time.time() - self.timeout
-                    if self.timeout <= 0 or mintime <= last_time:
-                        self._cache[cursor.dbname][key] = (value, time.time())
-                        result = value
-                        find = True
-            finally:
-                self._lock.release()
-
-            if not find:
-                # Work out new value, cache it and return it
-                # Should copy() this value to avoid futur modf of the cacle ?
-                result = function(self2, cursor, *args, **kwargs_origin)
-
-                self._lock.acquire()
-                try:
-                    self._cache[cursor.dbname][key] = (result, time.time())
-                finally:
-                    self._lock.release()
-            return result
-
-        cached_result.__doc__ = function.__doc__
-        return cached_result
-
-    @staticmethod
-    def clean(dbname):
-        if not CONFIG['multi_server']:
-            return
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        try:
-            cursor.execute('SELECT "timestamp", "name" FROM ir_cache')
-            timestamps = {}
-            for timestamp, name in cursor.fetchall():
-                timestamps[name] = timestamp
-        finally:
-            cursor.commit()
-            cursor.close()
-        for obj in Cache._cache_instance:
-            if obj._name in timestamps:
-                if not obj._timestamp or timestamps[obj._name] > obj._timestamp:
-                    obj._timestamp = timestamps[obj._name]
-                    obj._lock.acquire()
-                    try:
-                        obj._cache[dbname] = {}
-                    finally:
-                        obj._lock.release()
-
-    @staticmethod
-    def reset(dbname, name):
-        if not CONFIG['multi_server']:
-            return
-        Cache._resets_lock.acquire()
-        try:
-            Cache._resets.setdefault(dbname, set())
-            Cache._resets[dbname].add(name)
-        finally:
-            Cache._resets_lock.release()
-        return
-
-    @staticmethod
-    def resets(dbname):
-        if not CONFIG['multi_server']:
-            return
-        database = Database(dbname).connect()
-        cursor = database.cursor()
-        Cache._resets_lock.acquire()
-        Cache._resets.setdefault(dbname, set())
-        try:
-            for name in Cache._resets[dbname]:
-                cursor.execute('SELECT name FROM ir_cache WHERE name = %s',
-                            (name,))
-                if cursor.fetchone():
-                    cursor.execute('UPDATE ir_cache SET "timestamp" = %s '\
-                            'WHERE name = %s', (datetime.datetime.now(), name))
-                else:
-                    cursor.execute('INSERT INTO ir_cache ' \
-                            '("timestamp", "name") ' \
-                            'VALUES (%s, %s)', (datetime.datetime.now(), name))
-            Cache._resets[dbname].clear()
-        finally:
-            cursor.commit()
-            cursor.close()
-            Cache._resets_lock.release()
-
 def memoize(maxsize):
     """
     Decorator to 'memoize' a function - caching its results with a
@@ -603,3 +440,31 @@ def safe_eval(source, data=None):
         'round': round,
         'Decimal': Decimal,
         }}, data)
+
+def reduce_domain(domain):
+    '''
+    Reduce domain
+    '''
+    if not domain:
+        return []
+    operator = 'AND'
+    if isinstance(domain[0], basestring):
+        operator = domain[0]
+        domain = domain[1:]
+    result = [operator]
+    for arg in domain:
+        if (isinstance(arg, tuple) or
+                (isinstance(arg, list) and
+                    len(arg) > 2 and
+                    arg[1] in OPERATORS)):
+            #clause
+            result.append(arg)
+        elif isinstance(arg, list) and arg:
+            #sub-domain
+            sub_domain = reduce_domain(arg)
+            sub_operator = sub_domain[0]
+            if sub_operator == operator:
+                result.extend(sub_domain[1:])
+            else:
+                result.append(sub_domain)
+    return result
